@@ -1,5 +1,16 @@
-#ifndef __MAPPING_H
-#define __MAPPING_H
+//
+// Telescope - graphical task switcher
+//
+// (c) Ilya Skriblovsky, 2010
+// <Ilya.Skriblovsky@gmail.com>
+//
+
+// $Id: Mapping.h 94 2010-06-14 08:54:38Z mitrandir $
+
+// Mapping - a single key binding
+
+#ifndef __TELESCOPE__MAPPING_H
+#define __TELESCOPE__MAPPING_H
 
 #include <X11/Xlib.h>
 
