@@ -5,7 +5,7 @@
 // <Ilya.Skriblovsky@gmail.com>
 //
 
-// $Id: Thumbnail.cpp 100 2010-06-29 10:54:02Z mitrandir $
+// $Id: Thumbnail.cpp 111 2010-07-09 11:48:29Z mitrandir $
 
 #include "Thumbnail.h"
 
@@ -31,6 +31,7 @@ Thumbnail::Thumbnail(TeleWindow *teleWindow, Window clientWindow)
 #ifdef MAEMO4
     char *windowClass = XTools::windowClass_alloc(_clientWindow);
     _isOssoMediaPlayer = strcmp(windowClass, "mediaplayer-ui") == 0;
+    _isLiqBase = strncmp(windowClass, "liq", 3) == 0;
 #endif
 
 
@@ -539,3 +540,13 @@ bool Thumbnail::handleMousePress(int x, int y)
 
     return false;
 }
+
+
+bool Thumbnail::mustBeIconifiedBeforeTelescope()
+{
+#ifdef MAEMO4
+    return isOssoMediaPlayer() || isLiqBase();
+#else
+    return false;
+#endif
+}
