@@ -1,3 +1,12 @@
+//
+// Telescope - graphical task switcher
+//
+// (c) Ilya Skriblovsky, 2010
+// <Ilya.Skriblovsky@gmail.com>
+//
+
+// $Id: XTools.cpp 102 2010-06-29 13:20:56Z mitrandir $
+
 #include "XTools.h"
 
 #include <stdlib.h>
@@ -20,6 +29,8 @@ XTools::ErrorHandler XTools::_prevErrorHandler;
 Atom XTools::_NET_CLIENT_LIST;
 Atom XTools::_NET_WM_WINDOW_TYPE;
 Atom XTools::_NET_WM_WINDOW_TYPE_NORMAL;
+Atom XTools::_NET_WM_WINDOW_TYPE_SPLASH;
+Atom XTools::_NET_WM_WINDOW_TYPE_DESKTOP;
 Atom XTools::_NET_WM_NAME;
 Atom XTools::_NET_WM_STATE;
 Atom XTools::_NET_WM_STATE_FULLSCREEN;
@@ -40,6 +51,8 @@ void XTools::init(Display *dpy)
     INIT_ATOM(dpy, _NET_CLIENT_LIST);
     INIT_ATOM(dpy, _NET_WM_WINDOW_TYPE);
     INIT_ATOM(dpy, _NET_WM_WINDOW_TYPE_NORMAL);
+    INIT_ATOM(dpy, _NET_WM_WINDOW_TYPE_SPLASH);
+    INIT_ATOM(dpy, _NET_WM_WINDOW_TYPE_DESKTOP);
     INIT_ATOM(dpy, _NET_WM_NAME);
     INIT_ATOM(dpy, _NET_WM_STATE);
     INIT_ATOM(dpy, _NET_WM_STATE_FULLSCREEN);
@@ -55,7 +68,12 @@ void XTools::init(Display *dpy)
     _prevErrorHandler = XSetErrorHandler(errorHandler);
 }
 
-LinkedList<Window> XTools::windowList(Window rootWindow)
+Window XTools::rootWindow()
+{
+    return RootWindow(_dpy, DefaultScreen(_dpy));
+}
+
+LinkedList<Window> XTools::windowList(Window rootWindow, bool includeDesktop)
 {
     LinkedList<Window> list;
 
@@ -89,7 +107,10 @@ LinkedList<Window> XTools::windowList(Window rootWindow)
                 //bool ok = (items_read > 0 && *windowType == _NET_WM_WINDOW_TYPE_NORMAL);
                 bool ok = true;
                 if (items_read > 0 && *windowType != _NET_WM_WINDOW_TYPE_NORMAL)
-                    ok = false;
+                {
+                    if ((! includeDesktop) || *windowType != _NET_WM_WINDOW_TYPE_DESKTOP)
+                        ok = false;
+                }
 
                 XFree((unsigned char*)windowType);
 
@@ -129,14 +150,25 @@ char* XTools::windowTitle_alloc(Window window)
             return ret;
         }
         else
-        {
-            char *ret = (char*)malloc(1);
-            *ret = '\0';
-            return ret;
-        }
+            return strdup("");
     }
 }
 
+char* XTools::windowClass_alloc(Window window)
+{
+    XClassHint classHint;
+
+    if (XGetClassHint(_dpy, window, &classHint) != 0)
+    {
+        char *ret = strdup(classHint.res_name);
+        XFree(classHint.res_name);
+        XFree(classHint.res_class);
+        return ret;
+    }
+    else
+        return strdup("");
+}
+
 
 
 bool XTools::checkCompositeExtension()
@@ -240,25 +272,39 @@ int XTools::errorHandler(Display *display, XErrorEvent *event)
 }
 
 
-void XTools::showDesktop()
+void XTools::showDesktop(bool iconifyAll)
 {
-    XEvent event;
+    if (! iconifyAll)
+    {
+        // Right way if WM supports _NET_SHOWING_DESKTOP
 
-    Window root = DefaultRootWindow(_dpy);
+        XEvent event;
 
-    event.xclient.type = ClientMessage;
-    event.xclient.serial = 0;
-    event.xclient.send_event = True;
-    event.xclient.message_type = _NET_SHOWING_DESKTOP;
-    event.xclient.window = root;
-    event.xclient.format = 32;
-    event.xclient.data.l[0] = 1;
-    event.xclient.data.l[1] = 1;
-    event.xclient.data.l[2] = 0;
-    event.xclient.data.l[3] = 0;
-    event.xclient.data.l[4] = 0;
+        Window root = DefaultRootWindow(_dpy);
 
-    XSendEvent(_dpy, root, False, SubstructureRedirectMask | SubstructureNotifyMask, &event);
+        event.xclient.type = ClientMessage;
+        event.xclient.serial = 0;
+        event.xclient.send_event = True;
+        event.xclient.message_type = _NET_SHOWING_DESKTOP;
+        event.xclient.window = root;
+        event.xclient.format = 32;
+        event.xclient.data.l[0] = 1;
+        event.xclient.data.l[1] = 1;
+        event.xclient.data.l[2] = 0;
+        event.xclient.data.l[3] = 0;
+        event.xclient.data.l[4] = 0;
+
+        XSendEvent(_dpy, root, False, SubstructureRedirectMask | SubstructureNotifyMask, &event);
+    }
+    else
+    {
+        // Bad and ugly way -- minimizing all windows
+
+        LinkedList<Window> windows = windowList(RootWindow(_dpy, DefaultScreen(_dpy)));
+        for (LinkedList<Window>::Iter i = windows.head(); i; ++i)
+            if (! checkIfWindowMinimized(*i))
+                minimize(*i);
+    }
 }
 
 
@@ -281,7 +327,7 @@ bool XTools::checkIfWindowMinimized(Window window)
     if (status != Success)
         return false;
 
-    bool minimized = *property == 3;
+    bool minimized = *property == IconicState;
 
     XFree(property);
 
@@ -316,3 +362,53 @@ Window XTools::activeWindow()
 
     return result;
 }
+
+
+void XTools::minimize(Window window)
+{
+    XIconifyWindow(_dpy, window, DefaultScreen(_dpy));
+}
+
+
+
+XRenderPictFormat* XTools::xrenderFormat()
+{
+    static XRenderPictFormat *format = 0;
+
+    if (format == 0)
+    {
+        Visual *visual = DefaultVisual(_dpy, DefaultScreen(_dpy));
+        format = XRenderFindVisualFormat(_dpy, visual);
+    }
+
+    return format;
+}
+
+
+const XVisualInfo* XTools::rgbaVisual()
+{
+    static bool inited = false;
+    static XVisualInfo rgbaVisual;
+
+    if (inited == false)
+    {
+        int scr = DefaultScreen(_dpy);
+        if (XMatchVisualInfo(_dpy, scr, 32, TrueColor, &rgbaVisual) == 0)
+            fprintf(stderr, "Cannot find rgba visual\n");
+    }
+
+    return &rgbaVisual;
+}
+
+
+XRenderPictFormat* XTools::xrenderRGBAFormat()
+{
+    static XRenderPictFormat *rgbaFormat = 0;
+
+    if (rgbaFormat == 0)
+    {
+        rgbaFormat = XRenderFindVisualFormat(_dpy, rgbaVisual()->visual);
+    }
+
+    return rgbaFormat;
+}
